package cn.iocoder.yudao.module.delivery.controller.admin;

import cn.iocoder.yudao.framework.apollo.core.event.export.ReportCustomerAnalysisExcelExportEvent;
import cn.iocoder.yudao.framework.common.exception.ErrorCode;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.json.JsonUtils;
import cn.iocoder.yudao.framework.dict.core.dto.DictDataRespDTO;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.framework.redis.helper.RedisHelper;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.delivery.convert.CustomerAnalysis;
import cn.iocoder.yudao.module.delivery.entity.*;
import cn.iocoder.yudao.module.delivery.entity.orderdata.view_order_businesstime;
import cn.iocoder.yudao.module.delivery.service.CustomerAnalysisService;
import cn.iocoder.yudao.module.delivery.service.CustomerAnalysisService2;
import cn.iocoder.yudao.module.delivery.service.EcwReportPermissionService;
import cn.iocoder.yudao.module.delivery.service.view_order_businesstimeService;
import cn.iocoder.yudao.module.system.api.dict.DictDataApi;
import cn.iocoder.yudao.module.system.api.file.FileMakeApi;
import cn.iocoder.yudao.module.system.api.file.dto.FileMakeReqDTO;
import cn.iocoder.yudao.module.system.enums.download.DownloadTypeEnum;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.YearMonth;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;

/**
 * (CustomerAnalysis)客户分析报表
 *
 * @author
 * @since 2024-03-27 21:25:55
 */
@RestController
@RequestMapping("/Report/CustomerAnalysis")
public class CustomerAnalysisControl {

    @Autowired
    private CustomerAnalysisService customerAnalysisService;
    //报表权限信息
    @Resource
    private EcwReportPermissionService ecwReportPermissionService;
    @Resource
    private RedisHelper redisHelper;

    private List<view_order_businesstime> listBData = null;
    private List<CustomerAnalysisResp> objListResult = null;


    /*
     数据字典项
     */
    @Autowired
    private DictDataApi dictDataApi;

    @Resource
    private FileMakeApi fileMakeApi;

    @Resource
    private ApplicationContext applicationContext;


    /*
     客户分析列表
     xml动态查询参考网址
     https://blog.csdn.net/CYK_byte/article/details/128611104
     */
    @PostMapping("/getListPage")
    public CommonResult<PageResult<CustomerAnalysisResp>>
    getListPage(@RequestBody Map map) throws JsonProcessingException {
        //ecw_customer 的部门字段都为空 department 都为空，所以要关联客户经理所在的部门来查询
        //ecw_order_item  order_item_type 订单类型：1 普货 2 重货 3 泡货
        //ecw_order   order_type    订单类型：1 普货 2 重货 3 泡货
        // is_cargo_control 是否控货 1 控货，0不控货
        //同比增长速度=(本期发展水平-同期发展水平)/同期发展水平×100%
        //客户角色数据字典
        //客户表和数据字典表中的编码方式不一样，做关联查询有异常，为了不改变表的编码方式，在后台做转换

        //获取当前用户ID
        //先判断用户有什么报表权限
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        LambdaQueryWrapper<EcwReportPermission> lambdaQueryWrapper =
                new LambdaQueryWrapper();
        lambdaQueryWrapper.eq(EcwReportPermission::getUserId, loginUserId);
        List<EcwReportPermission> objList =
                ecwReportPermissionService.list(lambdaQueryWrapper);
        if (objList == null || objList.size() == 0) {
            ErrorCode eCode = new ErrorCode(2024041504,
                    "当前用户未分配报表权限，请联系系统管理员处理。");
            return error(eCode);
        }

        System.out.println(map);
        CustomerAnalysisReq query = JSON.parseObject(JSON.toJSONString(map),
                CustomerAnalysisReq.class);


        PageResult<CustomerAnalysisResp> pageResult =
                customerAnalysisService.GetDataResult(query);
        return success(pageResult);
    }

    @GetMapping("/exportExcel")
    @ApiOperation("导出客户分析数据")
    @OperateLog(type = EXPORT)
    public void export(HttpServletResponse response, @Valid CustomerAnalysisReq query) throws IOException {
        query = customerAnalysisService.getReq(query);
        query = customerAnalysisService.getDataWhere(query);
        System.out.println("处理后的查询条件：" + query);

        List<CustomerAnalysisResp> list =
                customerAnalysisService.GetDataListResult(query);
        List<CustomerAnalysisExcelResp> listExel =
                CustomerAnalysis.INSTANCE.convertList(list);
        //导出 Excel
        ExcelUtils.write(response, "客户分析报表.xls", "客户分析报表",
                CustomerAnalysisExcelResp.class, listExel);
    }

    @GetMapping("/exportExcel2")
    @ApiOperation("导出客户分析数据")
    @OperateLog(type = EXPORT)
    public CommonResult<Boolean> export2(HttpServletResponse response,
                                         @Valid CustomerAnalysisReq query) throws IOException {
        query = customerAnalysisService.getReq(query);
        query = customerAnalysisService.getDataWhere(query);
        System.out.println("处理后的查询条件：" + query);

        FileMakeReqDTO reqDTO = new FileMakeReqDTO();
        reqDTO.setType(DownloadTypeEnum.REPORT_CSSTOMER_ANALYSIS.getType());
        reqDTO.setName("客户分析报表");
        reqDTO.setFileSuffix("xlsx");
        reqDTO.setTemporaryFile(true);
        reqDTO.setUserType(2);
        reqDTO.setLang(I18nMessage.getLang());
        reqDTO.setRequestParams(JSONObject.toJSONString(query));
        fileMakeApi.sendFileMake(reqDTO);
        return CommonResult.success(true);

    }


}
