package cn.iocoder.yudao.module.delivery.controller.admin;

import cn.iocoder.yudao.framework.common.exception.ErrorCode;
import cn.iocoder.yudao.framework.redis.helper.RedisHelper;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.delivery.entity.EcwReportPermission;
import cn.iocoder.yudao.module.delivery.entity.EcwReportPermissionPageReq;
import cn.iocoder.yudao.module.delivery.entity.EcwReportPermissionResp;
import cn.iocoder.yudao.module.delivery.service.EcwReportPermissionService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import javax.validation.*;

import io.swagger.v3.oas.annotations.Operation;

import java.util.ArrayList;
import java.util.List;


/**
 * 报表权限(EcwReportPermission)表控制层
 *
 * @author
 * @since 2024-04-09 23:10:39
 */
@RestController
@RequestMapping("/Report/EcwReportPermission")
public class EcwReportPermissionController {
    /**
     * 服务对象
     */
    @Resource
    private EcwReportPermissionService ecwReportPermissionService;

    @Resource
    private RedisHelper redisHelper;

    private ErrorCode eCode;

    /*
     权限校验函数
     */
    private boolean isRight(EcwReportPermission ecwReportPermission) {
        //单表对象查询
        LambdaQueryWrapper<EcwReportPermission> lambdaQueryWrapper =
                new LambdaQueryWrapper();
        lambdaQueryWrapper.eq(EcwReportPermission::getUserId,
                ecwReportPermission.getUserId());

        if (ecwReportPermission.getPermissionFw() == 3) {
            lambdaQueryWrapper.eq(EcwReportPermission::getPermissionFw,
                    3);
            List<EcwReportPermission> objList = ecwReportPermissionService.list(lambdaQueryWrapper);
            if (objList != null && objList.size() > 0) {
                eCode = new ErrorCode(2024041501,
                        "用户的全公司权限已存在，不用再次添加");
                return false;
            }
        } else if (ecwReportPermission.getPermissionFw() == 1) {
            //个人
            lambdaQueryWrapper.eq(EcwReportPermission::getPermissionFw,
                    1);
            List<EcwReportPermission> objList = ecwReportPermissionService.list(lambdaQueryWrapper);
            if (objList != null && objList.size() > 0) {
                eCode = new ErrorCode(2024041502,
                        "用户的个人权限已存在，不用再次添加");
                return false;
            }
        } else if (ecwReportPermission.getPermissionFw() == 2) {
            //部门
            lambdaQueryWrapper.eq(EcwReportPermission::getPermissionFw,
                    2);
            lambdaQueryWrapper.eq(EcwReportPermission::getDeptId,
                    ecwReportPermission.getDeptId());
            List<EcwReportPermission> objList = ecwReportPermissionService.list(lambdaQueryWrapper);
            if (objList != null && objList.size() > 0) {
                eCode = new ErrorCode(2024041503,
                        "用户的部门权限已存在，不用再次添加");
                return false;
            }
        }
        return true;
    }

    /**
     * 新增数据
     *
     * @param ecwReportPermission 实体
     * @return 新增结果
     */
    @PostMapping("/create")
    public CommonResult<Long> create(@Valid @RequestBody EcwReportPermission ecwReportPermission) {

        boolean bR = isRight(ecwReportPermission);
        if (bR == false) {
            return error(eCode);
        }
        //System.out.println(ecwReportPermission);
        return success(ecwReportPermissionService.create(ecwReportPermission));
    }


    /**
     * 编辑数据
     *
     * @param ecwReportPermission 实体
     * @return 编辑结果
     */
    @PostMapping("/update")
    public CommonResult<Boolean> update(@RequestBody EcwReportPermission ecwReportPermission) {
        /*
        boolean bR = isRight(ecwReportPermission);
        if (bR == false) {
            return error(eCode);
        }*/
        ecwReportPermissionService.update(ecwReportPermission);
        return success(true);
    }


    /**
     * 删除数据
     *
     * @param id 主键
     * @return 删除是否成功
     */

    @GetMapping("/delete")
    public CommonResult<Boolean> delete(@RequestParam Long id) {
        ecwReportPermissionService.delete(id);
        return success(true);
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @GetMapping("/get")
    public CommonResult<EcwReportPermission> getModel(@RequestParam("id") Long id) {
        EcwReportPermission ecwReportPermission = ecwReportPermissionService.getModel(id);
        return success(ecwReportPermission);
    }

    /*
     获取当前用户的报表权限
     */
    @GetMapping("/getCurUserPermission")
    public CommonResult<EcwReportPermission> getCurUserPermission() {
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        LambdaQueryWrapper<EcwReportPermission> lambdaQueryWrapper =
                new LambdaQueryWrapper();
        lambdaQueryWrapper.eq(EcwReportPermission::getUserId, loginUserId);
        List<EcwReportPermission> objList = ecwReportPermissionService.list(lambdaQueryWrapper);
        if (objList == null || objList.size() == 0) {
            ErrorCode eCode = new ErrorCode(2024041504,
                    "当前用户未分配报表权限，请联系系统管理员处理。");
            return error(eCode);
        }
        return success(objList.get(0));
    }

    /**
     * 查询所有数据
     *
     * @return 实例对象集合
     */
    @GetMapping
    public CommonResult queryAll() {
        return success(this.ecwReportPermissionService.list());
    }

    /**
     * 分页查询
     *
     * @return 分页对象集合
     */
    @GetMapping("/page")
    @Operation(summary = "获取分页查询列表")
    public CommonResult<PageResult<EcwReportPermissionResp>> getListPage(@Valid EcwReportPermissionPageReq PageReq) {
        //myBatisPlus分页
        //return success(this.ecwReportPermissionService.getListPage(PageReq));
        //通过自定义SQL语句分页
        redisHelper.set("123","测试");
        List<String>l=new ArrayList<>();
        l.add("112");
        l.add("12444");
        redisHelper.set("myTest",l.toString());

        return success(this.ecwReportPermissionService.getListPageCustom(PageReq));
    }

}

