package cn.iocoder.yudao.module.delivery.controller.admin;


import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.json.JsonUtils;
import cn.iocoder.yudao.framework.dict.core.dto.DictDataRespDTO;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.module.delivery.entity.CustomerAnalysisReq;

import cn.iocoder.yudao.module.delivery.entity.SalesAnalysisExcelResp;
import cn.iocoder.yudao.module.delivery.entity.SalesAnalysisReq;
import cn.iocoder.yudao.module.delivery.entity.SalesAnalysisResp;
import cn.iocoder.yudao.module.delivery.entity.orderdata.StatisticalSummary;
import cn.iocoder.yudao.module.delivery.service.SalesAnalysisService;
import cn.iocoder.yudao.module.delivery.service.StatisticalSummaryService;
import cn.iocoder.yudao.module.system.api.dict.DictDataApi;
import cn.iocoder.yudao.module.system.api.file.FileMakeApi;
import cn.iocoder.yudao.module.system.api.file.dto.FileMakeReqDTO;
import cn.iocoder.yudao.module.system.enums.download.DownloadTypeEnum;
import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;

/**
 * (SalesAnalysis)销售分析报表
 *
 * @author
 * @since 2024-03-31
 */
@RestController
@RequestMapping("/Report/SalesAnalysis")
public class SalesAnalysisControl {

    @Autowired
    private SalesAnalysisService salesAnalysisService;

    @Resource
    private StatisticalSummaryService statisticalSummaryService;

    /*
    数据字典项
    */
    @Autowired
    private DictDataApi dictDataApi;

    @Resource
    private FileMakeApi fileMakeApi;


    /*
    客户分析列表
    */
    @GetMapping("/getListPage")
    public CommonResult<PageResult<SalesAnalysisResp>>
    getListPage(@Valid SalesAnalysisReq query) {
        System.out.println("订单分析查询条件：" + query);

        /*
        String jsonStr = JsonUtils.toJsonString(query);
        JsonUtils.SaveLog(jsonStr);
         */

        /*
        SalesAnalysisReq query = JSON.parseObject(JSON.toJSONString(map),
                SalesAnalysisReq.class);

         */
        PageResult<SalesAnalysisResp> pageResult =
                salesAnalysisService.getListPage(query);
        return success(pageResult);
    }

    /*
     获取销售统计数据
     */
    @GetMapping("/getStatisticalSummary")
    public CommonResult<StatisticalSummary> getStatisticalSummary(@Valid SalesAnalysisReq query) {
        return success(statisticalSummaryService.GetStatisticalSummary(query));
    }

    /*
     */
    @GetMapping("/exportExcel")
    @ApiOperation("导出销售分析数据")
    @OperateLog(type = EXPORT)
    public CommonResult<Boolean> export(HttpServletResponse response,
                                        @Valid SalesAnalysisReq Req)
            throws IOException {
        System.out.println("订单分析查询数据导出查询条件：" + Req);

        FileMakeReqDTO reqDTO = new FileMakeReqDTO();
        reqDTO.setType(DownloadTypeEnum.REPORT_SALES_ANALYSIS.getType());
        reqDTO.setName("销售分析数据");
        reqDTO.setFileSuffix("xlsx");
        reqDTO.setTemporaryFile(true);
        reqDTO.setUserType(2);
        reqDTO.setLang(I18nMessage.getLang());
        reqDTO.setRequestParams(JSONObject.toJSONString(Req));
        fileMakeApi.sendFileMake(reqDTO);
        return CommonResult.success(true);

        /*
        List<SalesAnalysisExcelResp> list = salesAnalysisService.getList(Req);
        ExcelUtils.write(response, "销售分析数据.xls", "销售分析数据",
                SalesAnalysisExcelResp.class, list);*/
    }
}
