package cn.iocoder.yudao.module.delivery.listener;

import cn.iocoder.yudao.framework.apollo.core.event.Order.CalculateOrderVValueEvent;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.delivery.entity.EcwVz;
//import cn.iocoder.yudao.module.delivery.entity.deptex.zTest;
import cn.iocoder.yudao.module.delivery.service.EcwVzService;
//import cn.iocoder.yudao.module.delivery.service.ZTestService;
import cn.iocoder.yudao.module.order.dal.dataobject.order.OrderDO;
import cn.iocoder.yudao.module.order.service.order.OrderService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Date;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.order.enums.ErrorCodeConstants.ORDER_NOT_EXISTS;
import static cn.iocoder.yudao.module.delivery.enums.ErrorCodeConstants.VCONFIG_NOT_EXISTS;

/*
 计算订单V值计算  海运已装柜，空运已入仓 计算V值
 */
@Component("CalculateOrderVValueListener")
@AllArgsConstructor
@Slf4j
public class CalculateOrderVValueListener {
    @Autowired
    private OrderService orderService;

    @Autowired
    private EcwVzService ecwVzService;

//    @Resource
//    private ZTestService zTestService;

    @EventListener(CalculateOrderVValueEvent.class)
    public void CalculateOrderVValuePushEvent(
            CalculateOrderVValueEvent event) {
//        zTest m = new zTest();
        String sMsg = "计算V值";
        OrderDO orderDO = orderService.getById(event.getOrderId());
        if (orderDO == null) {
            throw exception(ORDER_NOT_EXISTS);
        }

//        if (orderDO.getRucangTime() == null) {
//            sMsg += ",订单入仓时间为空，暂不处理。";
//            m.setTestname(sMsg);
//            m.setCreatedate(new Date());
//            zTestService.save(m);
//            return;
//        }

        //订单状态为已入仓之后的状态，其他状态是否排除，后续和客户确认，先按此逻辑处理
        int nS = orderDO.getStatus();
        if (nS >= 5) {
            sMsg += ",已入仓";
            //查询配置参数信息
            EcwVz temp = null;
            BigDecimal CalVvalue = new BigDecimal(0);
            if (orderDO.getTransportId() == 1) {
                //海运计算V值 修改查询条件
                temp = ecwVzService.getOne(new LambdaQueryWrapperX<EcwVz>()
                        .eqIfPresent(EcwVz::getFuhao, "M3"));
                if (temp == null) {
                    throw exception(VCONFIG_NOT_EXISTS);
                }
                if (orderDO.getSumVolume() == null ||
                        orderDO.getSumVolume().equals(new BigDecimal(0)) == true) {
                    CalVvalue = new BigDecimal(0);
                } else {
                    CalVvalue = new BigDecimal(temp.getVz()).divide(orderDO.getSumVolume());
                }
            } else if (orderDO.getTransportId() == 3) {
                //空运计算V值,修改查询条件
                temp = ecwVzService.getOne(new LambdaQueryWrapperX<EcwVz>()
                        .eqIfPresent(EcwVz::getFuhao, "KG"));
                if (temp == null) {
                    throw exception(VCONFIG_NOT_EXISTS);
                }
                if (orderDO.getSumWeight() == null ||
                        orderDO.getSumWeight().equals(new BigDecimal(0)) == true) {
                    CalVvalue = new BigDecimal(0);
                } else {
                    CalVvalue = new BigDecimal(temp.getVz()).divide(orderDO.getSumWeight());
                }
            }

            //计算业绩产生时间
            if (orderDO.getRucangTime() == null) {
                event.setYejiCreateDate(new Date());
            } else {
                event.setYejiCreateDate(orderDO.getRucangTime());
            }
            orderService.UpdateOrderVValue(event.getOrderId(), CalVvalue,
                    event.getYejiCreateDate());
        } else {
            sMsg += ",未入仓";
        }

//        m.setTestname(sMsg);
//        m.setCreatedate(new Date());
//        zTestService.save(m);

    }
}
